package CCfnX::DynamicValue {
  use Moose;
  extends 'Cfn::DynamicValue';
  # This is a stub class for backwards compatibility, as old code
  # references CCfnX::DynamicValue, but all the logic was incorporated
  # directly into the base Cfn module
}

1;
